@@ -1,13 +1,12 @@
-use rustyline::error::ReadlineError;
 use rustyline::validate::{ValidationContext, ValidationResult, Validator};
-use rustyline::Editor;
+use rustyline::{Editor, Result};
 use rustyline_derive::{Completer, Helper, Highlighter, Hinter};
 
 #[derive(Completer, Helper, Highlighter, Hinter)]
 struct InputValidator {}
 
 impl Validator for InputValidator {
-    fn validate(&self, ctx: &mut ValidationContext) -> Result<ValidationResult, ReadlineError> {
+    fn validate(&self, ctx: &mut ValidationContext) -> Result<ValidationResult> {
         use ValidationResult::{Incomplete, Invalid, Valid};
         let input = ctx.input();
         let result = if !input.starts_with("SELECT") {
@@ -21,7 +20,7 @@ impl Validator for InputValidator {
     }
 }
 
-fn main() -> rustyline::Result<()> {
+fn main() -> Result<()> {
     let h = InputValidator {};
     let mut rl = Editor::new();
     rl.set_helper(Some(h));
