@@ -2,7 +2,7 @@ use std::borrow::Cow::{self, Borrowed, Owned};
 
 use rustyline::config::Configurer;
 use rustyline::highlight::Highlighter;
-use rustyline::{ColorMode, Editor};
+use rustyline::{ColorMode, Editor, Result};
 use rustyline_derive::{Completer, Helper, Hinter, Validator};
 
 #[derive(Completer, Helper, Hinter, Validator)]
@@ -25,7 +25,7 @@ impl Highlighter for MaskingHighlighter {
     }
 }
 
-fn main() -> rustyline::Result<()> {
+fn main() -> Result<()> {
     println!("This is just a hack. Reading passwords securely requires more than that.");
     let h = MaskingHighlighter { masking: false };
     let mut rl = Editor::new();
