@@ -7,11 +7,6 @@ use crate::line_buffer::LineBuffer;
 use crate::{Context, Result};
 use memchr::memchr;
 
-// TODO: let the implementers choose/find word boundaries ???
-// (line, pos) is like (rl_line_buffer, rl_point) to make contextual completion
-// ("select t.na| from tbl as t")
-// TODO: make &self &mut self ???
-
 /// A completion candidate.
 pub trait Candidate {
     /// Text to display when listing alternatives.
@@ -30,6 +25,27 @@ impl Candidate for String {
     }
 }
 
+/// #[deprecated = "Unusable"]
+impl Candidate for str {
+    fn display(&self) -> &str {
+        self
+    }
+
+    fn replacement(&self) -> &str {
+        self
+    }
+}
+
+impl Candidate for &'_ str {
+    fn display(&self) -> &str {
+        self
+    }
+
+    fn replacement(&self) -> &str {
+        self
+    }
+}
+
 /// Completion candidate pair
 pub struct Pair {
     /// Text to display when listing alternatives.
@@ -48,18 +64,23 @@ impl Candidate for Pair {
     }
 }
 
+// TODO: let the implementers customize how the candidate(s) are displayed
+// https://github.com/kkawakam/rustyline/issues/302
+
 /// To be called for tab-completion.
 pub trait Completer {
     /// Specific completion candidate.
     type Candidate: Candidate;
 
+    // TODO: let the implementers choose/find word boundaries ??? => Lexer
+
     /// Takes the currently edited `line` with the cursor `pos`ition and
     /// returns the start position and the completion candidates for the
     /// partial word to be completed.
     ///
     /// ("ls /usr/loc", 11) => Ok((3, vec!["/usr/local/"]))
     fn complete(
-        &self,
+        &self, // FIXME should be `&mut self`
         line: &str,
         pos: usize,
         ctx: &Context<'_>,
@@ -198,11 +219,11 @@ impl FilenameCompleter {
                     )
                 }
             } else {
-                let (start, path) = extract_word(line, pos, ESCAPE_CHAR, &self.break_chars);
+                let (start, path) = extract_word(line, pos, ESCAPE_CHAR, self.break_chars);
                 let path = unescape(path, ESCAPE_CHAR);
                 (start, path, ESCAPE_CHAR, &self.break_chars, Quote::None)
             };
-        let mut matches = filename_complete(&path, esc_char, break_chars, quote)?;
+        let mut matches = filename_complete(&path, esc_char, break_chars, quote);
         #[allow(clippy::unnecessary_sort_by)]
         matches.sort_by(|a, b| a.display().cmp(b.display()));
         Ok((start, matches))
@@ -297,7 +318,7 @@ fn filename_complete(
     esc_char: Option<char>,
     break_chars: &[u8],
     quote: Quote,
-) -> Result<Vec<Pair>> {
+) -> Vec<Pair> {
     #[cfg(feature = "with-dirs")]
     use dirs_next::home_dir;
     use std::env::current_dir;
@@ -341,33 +362,31 @@ fn filename_complete(
 
     // if dir doesn't exist, then don't offer any completions
     if !dir.exists() {
-        return Ok(entries);
+        return entries;
     }
 
     // if any of the below IO operations have errors, just ignore them
     if let Ok(read_dir) = dir.read_dir() {
         let file_name = normalize(file_name);
-        for entry in read_dir {
-            if let Ok(entry) = entry {
-                if let Some(s) = entry.file_name().to_str() {
-                    let ns = normalize(s);
-                    if ns.starts_with(file_name.as_ref()) {
-                        if let Ok(metadata) = fs::metadata(entry.path()) {
-                            let mut path = String::from(dir_name) + s;
-                            if metadata.is_dir() {
-                                path.push(sep);
-                            }
-                            entries.push(Pair {
-                                display: String::from(s),
-                                replacement: escape(path, esc_char, break_chars, quote),
-                            });
-                        } // else ignore PermissionDenied
-                    }
+        for entry in read_dir.flatten() {
+            if let Some(s) = entry.file_name().to_str() {
+                let ns = normalize(s);
+                if ns.starts_with(file_name.as_ref()) {
+                    if let Ok(metadata) = fs::metadata(entry.path()) {
+                        let mut path = String::from(dir_name) + s;
+                        if metadata.is_dir() {
+                            path.push(sep);
+                        }
+                        entries.push(Pair {
+                            display: String::from(s),
+                            replacement: escape(path, esc_char, break_chars, quote),
+                        });
+                    } // else ignore PermissionDenied
                 }
             }
         }
     }
-    Ok(entries)
+    entries
 }
 
 #[cfg(any(windows, target_os = "macos"))]
@@ -425,7 +444,7 @@ pub fn longest_common_prefix<C: Candidate>(candidates: &[C]) -> Option<&str> {
     if candidates.is_empty() {
         return None;
     } else if candidates.len() == 1 {
-        return Some(&candidates[0].replacement());
+        return Some(candidates[0].replacement());
     }
     let mut longest_common_prefix = 0;
     'o: loop {
@@ -517,12 +536,12 @@ mod tests {
         let line = "ls '/usr/local/b";
         assert_eq!(
             (4, "/usr/local/b"),
-            super::extract_word(line, line.len(), Some('\\'), &break_chars)
+            super::extract_word(line, line.len(), Some('\\'), break_chars)
         );
         let line = "ls /User\\ Information";
         assert_eq!(
             (3, "/User\\ Information"),
-            super::extract_word(line, line.len(), Some('\\'), &break_chars)
+            super::extract_word(line, line.len(), Some('\\'), break_chars)
         );
     }
 
@@ -548,13 +567,13 @@ mod tests {
         let input = String::from("/usr/local/b");
         assert_eq!(
             input.clone(),
-            super::escape(input, Some('\\'), &break_chars, super::Quote::None)
+            super::escape(input, Some('\\'), break_chars, super::Quote::None)
         );
         let input = String::from("/User Information");
         let result = String::from("/User\\ Information");
         assert_eq!(
             result,
-            super::escape(input, Some('\\'), &break_chars, super::Quote::None)
+            super::escape(input, Some('\\'), break_chars, super::Quote::None)
         );
     }
 
