@@ -9,7 +9,8 @@ use unicode_width::UnicodeWidthChar;
 
 use super::{Context, Helper, Result};
 use crate::highlight::Highlighter;
-use crate::history::Direction;
+use crate::hint::Hint;
+use crate::history::SearchDirection;
 use crate::keymap::{Anchor, At, CharSearch, Cmd, Movement, RepeatCount, Word};
 use crate::keymap::{InputState, Invoke, Refresher};
 use crate::layout::{Layout, Position};
@@ -30,9 +31,9 @@ pub struct State<'out, 'prompt, H: Helper> {
     byte_buffer: [u8; 4],
     pub changes: Rc<RefCell<Changeset>>, // changes to line, for undo/redo
     pub helper: Option<&'out H>,
-    pub ctx: Context<'out>,   // Give access to history for `hinter`
-    pub hint: Option<String>, // last hint displayed
-    highlight_char: bool,     // `true` if a char has been highlighted
+    pub ctx: Context<'out>,          // Give access to history for `hinter`
+    pub hint: Option<Box<dyn Hint>>, // last hint displayed
+    highlight_char: bool,            // `true` if a char has been highlighted
 }
 
 enum Info<'m> {
@@ -129,6 +130,15 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         Ok(())
     }
 
+    pub fn move_cursor_to_end(&mut self) -> Result<()> {
+        if self.layout.cursor == self.layout.end {
+            return Ok(());
+        }
+        self.out.move_cursor(self.layout.cursor, self.layout.end)?;
+        self.layout.cursor = self.layout.end;
+        Ok(())
+    }
+
     pub fn move_cursor_at_leftmost(&mut self, rdr: &mut <Terminal as Term>::Reader) -> Result<()> {
         self.out.move_cursor_at_leftmost(rdr)
     }
@@ -142,7 +152,7 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
     ) -> Result<()> {
         let info = match info {
             Info::NoHint => None,
-            Info::Hint => self.hint.as_deref(),
+            Info::Hint => self.hint.as_ref().map(|h| h.display()),
             Info::Msg(msg) => msg,
         };
         let highlighter = if self.out.colors_enabled() {
@@ -173,9 +183,12 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
     pub fn hint(&mut self) {
         if let Some(hinter) = self.helper {
             let hint = hinter.hint(self.line.as_str(), self.line.pos(), &self.ctx);
-            self.hint = hint;
+            self.hint = match hint {
+                Some(val) if !val.display().is_empty() => Some(Box::new(val) as Box<dyn Hint>),
+                _ => None,
+            };
         } else {
-            self.hint = None
+            self.hint = None;
         }
     }
 
@@ -201,32 +214,30 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         self.layout.default_prompt
     }
 
-    pub fn validate(&mut self) -> Result<bool> {
+    pub fn validate(&mut self) -> Result<ValidationResult> {
         if let Some(validator) = self.helper {
             self.changes.borrow_mut().begin();
             let result = validator.validate(&mut ValidationContext::new(self))?;
             let corrected = self.changes.borrow_mut().end();
-            let validated = match result {
-                ValidationResult::Incomplete => false,
-                ValidationResult::Valid(msg) => {
+            match result {
+                ValidationResult::Incomplete => {}
+                ValidationResult::Valid(ref msg) => {
                     // Accept the line regardless of where the cursor is.
                     if corrected || self.has_hint() || msg.is_some() {
                         // Force a refresh without hints to leave the previous
                         // line as the user typed it after a newline.
-                        self.refresh_line_with_msg(msg)?;
+                        self.refresh_line_with_msg(msg.as_deref())?;
                     }
-                    true
                 }
-                ValidationResult::Invalid(msg) => {
+                ValidationResult::Invalid(ref msg) => {
                     if corrected || self.has_hint() || msg.is_some() {
-                        self.refresh_line_with_msg(msg)?;
+                        self.refresh_line_with_msg(msg.as_deref())?;
                     }
-                    false
                 }
-            };
-            Ok(validated)
+            }
+            Ok(result)
         } else {
-            Ok(true)
+            Ok(ValidationResult::Valid(None))
         }
     }
 }
@@ -245,11 +256,11 @@ impl<'out, 'prompt, H: Helper> Refresher for State<'out, 'prompt, H> {
         self.refresh(self.prompt, prompt_size, true, Info::Hint)
     }
 
-    fn refresh_line_with_msg(&mut self, msg: Option<String>) -> Result<()> {
+    fn refresh_line_with_msg(&mut self, msg: Option<&str>) -> Result<()> {
         let prompt_size = self.prompt_size;
         self.hint = None;
         self.highlight_char();
-        self.refresh(self.prompt, prompt_size, true, Info::Msg(msg.as_deref()))
+        self.refresh(self.prompt, prompt_size, true, Info::Msg(msg))
     }
 
     fn refresh_prompt_and_line(&mut self, prompt: &str) -> Result<()> {
@@ -278,6 +289,18 @@ impl<'out, 'prompt, H: Helper> Refresher for State<'out, 'prompt, H> {
     fn has_hint(&self) -> bool {
         self.hint.is_some()
     }
+
+    fn hint_text(&self) -> Option<&str> {
+        self.hint.as_ref().and_then(|hint| hint.completion())
+    }
+
+    fn line(&self) -> &str {
+        self.line.as_str()
+    }
+
+    fn pos(&self) -> usize {
+        self.line.pos()
+    }
 }
 
 impl<'out, 'prompt, H: Helper> fmt::Debug for State<'out, 'prompt, H> {
@@ -471,14 +494,6 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         self.refresh_line()
     }
 
-    pub fn edit_delete(&mut self, n: RepeatCount) -> Result<()> {
-        if self.line.delete(n).is_some() {
-            self.refresh_line()
-        } else {
-            Ok(())
-        }
-    }
-
     /// Exchange the char before cursor with the character at cursor.
     pub fn edit_transpose_chars(&mut self) -> Result<()> {
         self.changes.borrow_mut().begin();
@@ -592,30 +607,29 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
     }
 
     // Non-incremental, anchored search
-    pub fn edit_history_search(&mut self, dir: Direction) -> Result<()> {
+    pub fn edit_history_search(&mut self, dir: SearchDirection) -> Result<()> {
         let history = self.ctx.history;
         if history.is_empty() {
             return self.out.beep();
         }
-        if self.ctx.history_index == history.len() && dir == Direction::Forward
-            || self.ctx.history_index == 0 && dir == Direction::Reverse
+        if self.ctx.history_index == history.len() && dir == SearchDirection::Forward
+            || self.ctx.history_index == 0 && dir == SearchDirection::Reverse
         {
             return self.out.beep();
         }
-        if dir == Direction::Reverse {
+        if dir == SearchDirection::Reverse {
             self.ctx.history_index -= 1;
         } else {
             self.ctx.history_index += 1;
         }
-        if let Some(history_index) = history.starts_with(
+        if let Some(sr) = history.starts_with(
             &self.line.as_str()[..self.line.pos()],
             self.ctx.history_index,
             dir,
         ) {
-            self.ctx.history_index = history_index;
-            let buf = history.get(history_index).unwrap();
+            self.ctx.history_index = sr.idx;
             self.changes.borrow_mut().begin();
-            self.line.update(buf, buf.len());
+            self.line.update(sr.entry, sr.pos);
             self.changes.borrow_mut().end();
             self.refresh_line()
         } else {
@@ -652,6 +666,15 @@ impl<'out, 'prompt, H: Helper> State<'out, 'prompt, H> {
         }
         self.refresh_line()
     }
+
+    /// Change the indentation of the lines covered by movement
+    pub fn edit_indent(&mut self, mvt: &Movement, amount: usize, dedent: bool) -> Result<()> {
+        if self.line.indent(mvt, amount, dedent) {
+            self.refresh_line()
+        } else {
+            Ok(())
+        }
+    }
 }
 
 #[cfg(test)]
@@ -673,7 +696,7 @@ pub fn init_state<'out, H: Helper>(
         changes: Rc::new(RefCell::new(Changeset::new())),
         helper,
         ctx: Context::new(history),
-        hint: Some("hint".to_owned()),
+        hint: Some(Box::new("hint".to_owned())),
         highlight_char: false,
     }
 }
