@@ -42,7 +42,7 @@ pub trait Highlighter {
     /// Currently, used only with `CompletionType::List`.
     fn highlight_candidate<'c>(
         &self,
-        candidate: &'c str,
+        candidate: &'c str, // FIXME should be Completer::Candidate
         completion: CompletionType,
     ) -> Cow<'c, str> {
         let _ = completion;
@@ -94,6 +94,8 @@ impl<'r, H: ?Sized + Highlighter> Highlighter for &'r H {
 const OPENS: &[u8; 3] = b"{[(";
 const CLOSES: &[u8; 3] = b"}])";
 
+/// TODO versus https://python-prompt-toolkit.readthedocs.io/en/master/pages/reference.html?highlight=HighlightMatchingBracketProcessor#prompt_toolkit.layout.processors.HighlightMatchingBracketProcessor
+
 /// Highlight matching bracket when typed or cursor moved on.
 #[derive(Default)]
 pub struct MatchingBracketHighlighter {
@@ -193,17 +195,13 @@ fn check_bracket(line: &str, pos: usize) -> Option<(u8, usize)> {
         loop {
             let b = line.as_bytes()[pos];
             if is_close_bracket(b) {
-                if pos == 0 {
-                    return None;
-                } else {
-                    return Some((b, pos));
-                }
+                return if pos == 0 { None } else { Some((b, pos)) };
             } else if is_open_bracket(b) {
-                if pos + 1 == line.len() {
-                    return None;
+                return if pos + 1 == line.len() {
+                    None
                 } else {
-                    return Some((b, pos));
-                }
+                    Some((b, pos))
+                };
             } else if under_cursor && pos > 0 {
                 under_cursor = false;
                 pos -= 1; // or before cursor
