@@ -1,6 +1,7 @@
 //! Line buffer with current cursor position
 use crate::keymap::{At, CharSearch, Movement, RepeatCount, Word};
 use std::cell::RefCell;
+use std::cmp::min;
 use std::fmt;
 use std::iter;
 use std::ops::{Deref, Index, Range};
@@ -11,16 +12,17 @@ use unicode_segmentation::UnicodeSegmentation;
 
 /// Default maximum buffer size for the line read
 pub(crate) const MAX_LINE: usize = 4096;
+pub(crate) const INDENT: &str = "                                ";
 
 /// Word's case change
 #[derive(Clone, Copy)]
 pub enum WordAction {
     /// Capitalize word
-    CAPITALIZE,
+    Capitalize,
     /// lowercase word
-    LOWERCASE,
+    Lowercase,
     /// uppercase word
-    UPPERCASE,
+    Uppercase,
 }
 
 /// Delete (kill) direction
@@ -50,6 +52,8 @@ pub(crate) trait ChangeListener: DeleteListener {
     fn replace(&mut self, idx: usize, old: &str, new: &str);
 }
 
+// TODO split / cache lines ?
+
 /// Represent the current input (text and cursor position).
 ///
 /// The methods do text manipulations or/and cursor movements.
@@ -256,7 +260,7 @@ impl LineBuffer {
         if n == 1 {
             self.insert_str(pos, text);
         } else {
-            let text = iter::repeat(text).take(n).collect::<String>();
+            let text = text.repeat(n);
             self.insert_str(pos, &text);
         }
         self.pos += shift;
@@ -692,23 +696,19 @@ impl LineBuffer {
                 }
             }
         };
-        if let Some(pos) = search_result {
-            Some(match cs {
-                CharSearch::Backward(_) => pos,
-                CharSearch::BackwardAfter(c) => pos + c.len_utf8(),
-                CharSearch::Forward(_) => shift + pos,
-                CharSearch::ForwardBefore(_) => {
-                    shift + pos
-                        - self.buf[..shift + pos]
-                            .chars()
-                            .next_back()
-                            .unwrap()
-                            .len_utf8()
-                }
-            })
-        } else {
-            None
-        }
+        search_result.map(|pos| match cs {
+            CharSearch::Backward(_) => pos,
+            CharSearch::BackwardAfter(c) => pos + c.len_utf8(),
+            CharSearch::Forward(_) => shift + pos,
+            CharSearch::ForwardBefore(_) => {
+                shift + pos
+                    - self.buf[..shift + pos]
+                        .chars()
+                        .next_back()
+                        .unwrap()
+                        .len_utf8()
+            }
+        })
     }
 
     /// Move cursor to the matching character position.
@@ -790,13 +790,13 @@ impl LineBuffer {
                     .drain(start..end, Direction::default())
                     .collect::<String>();
                 let result = match a {
-                    WordAction::CAPITALIZE => {
+                    WordAction::Capitalize => {
                         let ch = (&word).graphemes(true).next().unwrap();
                         let cap = ch.to_uppercase();
                         cap + &word[ch.len()..].to_lowercase()
                     }
-                    WordAction::LOWERCASE => word.to_lowercase(),
-                    WordAction::UPPERCASE => word.to_uppercase(),
+                    WordAction::Lowercase => word.to_lowercase(),
+                    WordAction::Uppercase => word.to_uppercase(),
                 };
                 self.insert_str(start, &result);
                 self.pos = start + result.len();
@@ -921,10 +921,9 @@ impl LineBuffer {
             Movement::ViFirstPrint => {
                 if self.pos == 0 {
                     None
-                } else if let Some(pos) = self.next_word_pos(0, At::Start, Word::Big, 1) {
-                    Some(self.buf[pos..self.pos].to_owned())
                 } else {
-                    None
+                    self.next_word_pos(0, At::Start, Word::Big, 1)
+                        .map(|pos| self.buf[pos..self.pos].to_owned())
                 }
             }
             Movement::EndOfLine => {
@@ -956,51 +955,31 @@ impl LineBuffer {
                     Some(self.buf[..self.pos].to_owned())
                 }
             }
-            Movement::BackwardWord(n, word_def) => {
-                if let Some(pos) = self.prev_word_pos(self.pos, word_def, n) {
-                    Some(self.buf[pos..self.pos].to_owned())
-                } else {
-                    None
-                }
-            }
-            Movement::ForwardWord(n, at, word_def) => {
-                if let Some(pos) = self.next_word_pos(self.pos, at, word_def, n) {
-                    Some(self.buf[self.pos..pos].to_owned())
-                } else {
-                    None
-                }
-            }
+            Movement::BackwardWord(n, word_def) => self
+                .prev_word_pos(self.pos, word_def, n)
+                .map(|pos| self.buf[pos..self.pos].to_owned()),
+            Movement::ForwardWord(n, at, word_def) => self
+                .next_word_pos(self.pos, at, word_def, n)
+                .map(|pos| self.buf[self.pos..pos].to_owned()),
             Movement::ViCharSearch(n, cs) => {
                 let search_result = match cs {
                     CharSearch::ForwardBefore(c) => self.search_char_pos(CharSearch::Forward(c), n),
                     _ => self.search_char_pos(cs, n),
                 };
-                if let Some(pos) = search_result {
-                    Some(match cs {
-                        CharSearch::Backward(_) | CharSearch::BackwardAfter(_) => {
-                            self.buf[pos..self.pos].to_owned()
-                        }
-                        CharSearch::ForwardBefore(_) => self.buf[self.pos..pos].to_owned(),
-                        CharSearch::Forward(c) => self.buf[self.pos..pos + c.len_utf8()].to_owned(),
-                    })
-                } else {
-                    None
-                }
-            }
-            Movement::BackwardChar(n) => {
-                if let Some(pos) = self.prev_pos(n) {
-                    Some(self.buf[pos..self.pos].to_owned())
-                } else {
-                    None
-                }
-            }
-            Movement::ForwardChar(n) => {
-                if let Some(pos) = self.next_pos(n) {
-                    Some(self.buf[self.pos..pos].to_owned())
-                } else {
-                    None
-                }
+                search_result.map(|pos| match cs {
+                    CharSearch::Backward(_) | CharSearch::BackwardAfter(_) => {
+                        self.buf[pos..self.pos].to_owned()
+                    }
+                    CharSearch::ForwardBefore(_) => self.buf[self.pos..pos].to_owned(),
+                    CharSearch::Forward(c) => self.buf[self.pos..pos + c.len_utf8()].to_owned(),
+                })
             }
+            Movement::BackwardChar(n) => self
+                .prev_pos(n)
+                .map(|pos| self.buf[pos..self.pos].to_owned()),
+            Movement::ForwardChar(n) => self
+                .next_pos(n)
+                .map(|pos| self.buf[self.pos..pos].to_owned()),
             Movement::LineUp(n) => {
                 if let Some((start, end)) = self.n_lines_up(n) {
                     Some(self.buf[start..end].to_owned())
@@ -1020,10 +999,7 @@ impl LineBuffer {
 
     /// Kill range specified by `mvt`.
     pub fn kill(&mut self, mvt: &Movement) -> bool {
-        let notify = match *mvt {
-            Movement::ForwardChar(_) | Movement::BackwardChar(_) => false,
-            _ => true,
-        };
+        let notify = !matches!(*mvt, Movement::ForwardChar(_) | Movement::BackwardChar(_));
         if notify {
             if let Some(dl) = self.dl.as_ref() {
                 let mut dl = dl.lock().unwrap();
@@ -1100,6 +1076,68 @@ impl LineBuffer {
         }
         killed
     }
+
+    /// Indent range specified by `mvt`.
+    pub fn indent(&mut self, mvt: &Movement, amount: usize, dedent: bool) -> bool {
+        let pair = match *mvt {
+            // All inline operators are the same: indent current line
+            Movement::WholeLine
+            | Movement::BeginningOfLine
+            | Movement::ViFirstPrint
+            | Movement::EndOfLine
+            | Movement::BackwardChar(..)
+            | Movement::ForwardChar(..)
+            | Movement::ViCharSearch(..) => Some((self.pos, self.pos)),
+            Movement::EndOfBuffer => Some((self.pos, self.buf.len())),
+            Movement::WholeBuffer => Some((0, self.buf.len())),
+            Movement::BeginningOfBuffer => Some((0, self.pos)),
+            Movement::BackwardWord(n, word_def) => self
+                .prev_word_pos(self.pos, word_def, n)
+                .map(|pos| (pos, self.pos)),
+            Movement::ForwardWord(n, at, word_def) => self
+                .next_word_pos(self.pos, at, word_def, n)
+                .map(|pos| (self.pos, pos)),
+            Movement::LineUp(n) => self.n_lines_up(n),
+            Movement::LineDown(n) => self.n_lines_down(n),
+        };
+        let (start, end) = pair.unwrap_or((self.pos, self.pos));
+        let start = self.buf[..start]
+            .rfind('\n')
+            .map(|pos| pos + 1)
+            .unwrap_or(0);
+        let end = self.buf[end..]
+            .rfind('\n')
+            .map(|pos| end + pos)
+            .unwrap_or_else(|| self.buf.len());
+        let mut index = start;
+        if dedent {
+            for line in self.buf[start..end].to_string().split('\n') {
+                let max = line.len() - line.trim_start().len();
+                let deleting = min(max, amount);
+                self.drain(index..index + deleting, Default::default());
+                if self.pos >= index {
+                    if self.pos.saturating_sub(index) < deleting {
+                        // don't wrap into the previous line
+                        self.pos = index;
+                    } else {
+                        self.pos -= deleting;
+                    }
+                }
+                index += line.len() + 1 - deleting;
+            }
+        } else {
+            for line in self.buf[start..end].to_string().split('\n') {
+                for off in (0..amount).step_by(INDENT.len()) {
+                    self.insert_str(index, &INDENT[..min(amount - off, INDENT.len())]);
+                }
+                if self.pos >= index {
+                    self.pos += amount;
+                }
+                index += amount + line.len() + 1;
+            }
+        }
+        true
+    }
 }
 
 impl Deref for LineBuffer {
@@ -1203,18 +1241,18 @@ mod test {
         let push = s.insert('α', 1).unwrap();
         assert_eq!("α", s.buf);
         assert_eq!(2, s.pos);
-        assert_eq!(true, push);
+        assert!(push);
 
         let push = s.insert('ß', 1).unwrap();
         assert_eq!("αß", s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, push);
+        assert!(push);
 
         s.pos = 0;
         let push = s.insert('γ', 1).unwrap();
         assert_eq!("γαß", s.buf);
         assert_eq!(2, s.pos);
-        assert_eq!(false, push);
+        assert!(!push);
     }
 
     #[test]
@@ -1242,22 +1280,22 @@ mod test {
         let ok = s.move_backward(1);
         assert_eq!("αß", s.buf);
         assert_eq!(2, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_forward(1);
         assert_eq!("αß", s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_home();
         assert_eq!("αß", s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_end();
         assert_eq!("αß", s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
     }
 
     #[test]
@@ -1267,22 +1305,22 @@ mod test {
         let ok = s.move_home();
         assert_eq!(text, s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_home();
         assert_eq!(text, s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(false, ok);
+        assert!(!ok);
 
         let ok = s.move_end();
         assert_eq!(text, s.buf);
         assert_eq!(11, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_end();
         assert_eq!(text, s.buf);
         assert_eq!(11, s.pos);
-        assert_eq!(false, ok);
+        assert!(!ok);
     }
 
     #[test]
@@ -1292,22 +1330,22 @@ mod test {
         let ok = s.move_buffer_start();
         assert_eq!(text, s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_buffer_start();
         assert_eq!(text, s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(false, ok);
+        assert!(!ok);
 
         let ok = s.move_buffer_end();
         assert_eq!(text, s.buf);
         assert_eq!(text.len(), s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         let ok = s.move_buffer_end();
         assert_eq!(text, s.buf);
         assert_eq!(text.len(), s.pos);
-        assert_eq!(false, ok);
+        assert!(!ok);
     }
 
     #[test]
@@ -1315,11 +1353,11 @@ mod test {
         let mut s = LineBuffer::init("ag̈", 4, None);
         assert_eq!(4, s.len());
         let ok = s.move_backward(1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(1, s.pos);
 
         let ok = s.move_forward(1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(4, s.pos);
     }
 
@@ -1335,7 +1373,7 @@ mod test {
         let ok = s.backspace(1);
         assert_eq!("", s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("α");
     }
 
@@ -1346,14 +1384,14 @@ mod test {
         let ok = s.kill_line();
         assert_eq!("αßγ", s.buf);
         assert_eq!(6, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("δε");
 
         s.pos = 4;
         let ok = s.discard_line();
         assert_eq!("γ", s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("αß");
     }
 
@@ -1365,24 +1403,24 @@ mod test {
         let ok = s.kill_line();
         assert_eq!("αß\nγ\nε f4", s.buf);
         assert_eq!(7, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("δ 12");
 
         let ok = s.kill_line();
         assert_eq!("αß\nγε f4", s.buf);
         assert_eq!(7, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("\n");
 
         let ok = s.kill_line();
         assert_eq!("αß\nγ", s.buf);
         assert_eq!(7, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ε f4");
 
         let ok = s.kill_line();
         assert_eq!(7, s.pos);
-        assert_eq!(false, ok);
+        assert!(!ok);
     }
 
     #[test]
@@ -1393,24 +1431,24 @@ mod test {
         let ok = s.discard_line();
         assert_eq!("αß\nδε", s.buf);
         assert_eq!(5, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("c γ");
 
         let ok = s.discard_line();
         assert_eq!("αßδε", s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("\n");
 
         let ok = s.discard_line();
         assert_eq!("δε", s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("αß");
 
         let ok = s.discard_line();
         assert_eq!(0, s.pos);
-        assert_eq!(false, ok);
+        assert!(!ok);
     }
 
     #[test]
@@ -1419,21 +1457,21 @@ mod test {
         let ok = s.transpose_chars();
         assert_eq!("ßac", s.buf);
         assert_eq!(3, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         s.buf = String::from("aßc");
         s.pos = 3;
         let ok = s.transpose_chars();
         assert_eq!("acß", s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
 
         s.buf = String::from("aßc");
         s.pos = 4;
         let ok = s.transpose_chars();
         assert_eq!("acß", s.buf);
         assert_eq!(4, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
     }
 
     #[test]
@@ -1503,17 +1541,17 @@ mod test {
     fn move_to_forward() {
         let mut s = LineBuffer::init("αßγδε", 2, None);
         let ok = s.move_to(CharSearch::ForwardBefore('ε'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(6, s.pos);
 
         let mut s = LineBuffer::init("αßγδε", 2, None);
         let ok = s.move_to(CharSearch::Forward('ε'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(8, s.pos);
 
         let mut s = LineBuffer::init("αßγδε", 2, None);
         let ok = s.move_to(CharSearch::Forward('ε'), 10);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(8, s.pos);
     }
 
@@ -1521,12 +1559,12 @@ mod test {
     fn move_to_backward() {
         let mut s = LineBuffer::init("αßγδε", 8, None);
         let ok = s.move_to(CharSearch::BackwardAfter('ß'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(4, s.pos);
 
         let mut s = LineBuffer::init("αßγδε", 8, None);
         let ok = s.move_to(CharSearch::Backward('ß'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(2, s.pos);
     }
 
@@ -1537,7 +1575,7 @@ mod test {
         let ok = s.delete_prev_word(Word::Big, 1);
         assert_eq!("a c", s.buf);
         assert_eq!(2, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ß  ");
     }
 
@@ -1546,20 +1584,20 @@ mod test {
         let mut s = LineBuffer::init("a ß  c", 1, None); // after 'a'
         let ok = s.move_to_next_word(At::AfterEnd, Word::Emacs, 1);
         assert_eq!("a ß  c", s.buf);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(4, s.pos); // after 'ß'
 
         let ok = s.move_to_next_word(At::AfterEnd, Word::Emacs, 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(7, s.pos); // after 'c'
 
         s.move_home();
         let ok = s.move_to_next_word(At::AfterEnd, Word::Emacs, 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(1, s.pos); // after 'a'
 
         let ok = s.move_to_next_word(At::AfterEnd, Word::Emacs, 2);
-        assert_eq!(true, ok);
+        assert!(ok);
         assert_eq!(7, s.pos); // after 'c'
     }
 
@@ -1569,7 +1607,7 @@ mod test {
         let ok = s.move_to_next_word(At::BeforeEnd, Word::Vi, 1);
         assert_eq!("a ßeta  c", s.buf);
         assert_eq!(6, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
     }
 
     #[test]
@@ -1622,7 +1660,7 @@ mod test {
         let ok = s.move_to_next_word(At::Start, Word::Emacs, 1);
         assert_eq!("a ß  c", s.buf);
         assert_eq!(6, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
     }
 
     #[test]
@@ -1676,14 +1714,14 @@ mod test {
         let ok = s.delete_word(At::AfterEnd, Word::Emacs, 1);
         assert_eq!("a  c", s.buf);
         assert_eq!(1, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq(" ß");
 
         let mut s = LineBuffer::init("test", 0, Some(cl.clone()));
         let ok = s.delete_word(At::AfterEnd, Word::Vi, 1);
         assert_eq!("", s.buf);
         assert_eq!(0, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("test");
     }
 
@@ -1694,7 +1732,7 @@ mod test {
         let ok = s.delete_word(At::Start, Word::Emacs, 1);
         assert_eq!("a c", s.buf);
         assert_eq!(2, s.pos);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ß  ");
     }
 
@@ -1703,14 +1741,14 @@ mod test {
         let cl = Listener::new();
         let mut s = LineBuffer::init("αßγδε", 2, Some(cl.clone()));
         let ok = s.delete_to(CharSearch::ForwardBefore('ε'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ßγδ");
         assert_eq!("αε", s.buf);
         assert_eq!(2, s.pos);
 
         let mut s = LineBuffer::init("αßγδε", 2, Some(cl.clone()));
         let ok = s.delete_to(CharSearch::Forward('ε'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ßγδε");
         assert_eq!("α", s.buf);
         assert_eq!(2, s.pos);
@@ -1721,14 +1759,14 @@ mod test {
         let cl = Listener::new();
         let mut s = LineBuffer::init("αßγδε", 8, Some(cl.clone()));
         let ok = s.delete_to(CharSearch::BackwardAfter('α'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ßγδ");
         assert_eq!("αε", s.buf);
         assert_eq!(2, s.pos);
 
         let mut s = LineBuffer::init("αßγδε", 8, Some(cl.clone()));
         let ok = s.delete_to(CharSearch::Backward('ß'), 1);
-        assert_eq!(true, ok);
+        assert!(ok);
         cl.borrow().assert_deleted_str_eq("ßγδ");
         assert_eq!("αε", s.buf);
         assert_eq!(2, s.pos);
@@ -1737,22 +1775,22 @@ mod test {
     #[test]
     fn edit_word() {
         let mut s = LineBuffer::init("a ßeta  c", 1, None);
-        assert!(s.edit_word(WordAction::UPPERCASE));
+        assert!(s.edit_word(WordAction::Uppercase));
         assert_eq!("a SSETA  c", s.buf);
         assert_eq!(7, s.pos);
 
         let mut s = LineBuffer::init("a ßetA  c", 1, None);
-        assert!(s.edit_word(WordAction::LOWERCASE));
+        assert!(s.edit_word(WordAction::Lowercase));
         assert_eq!("a ßeta  c", s.buf);
         assert_eq!(7, s.pos);
 
         let mut s = LineBuffer::init("a ßETA  c", 1, None);
-        assert!(s.edit_word(WordAction::CAPITALIZE));
+        assert!(s.edit_word(WordAction::Capitalize));
         assert_eq!("a SSeta  c", s.buf);
         assert_eq!(7, s.pos);
 
         let mut s = LineBuffer::init("test", 1, None);
-        assert!(s.edit_word(WordAction::CAPITALIZE));
+        assert!(s.edit_word(WordAction::Capitalize));
         assert_eq!("tEst", s.buf);
         assert_eq!(4, s.pos);
     }
