@@ -4,10 +4,10 @@ use unicode_width::UnicodeWidthStr;
 
 use crate::config::{BellStyle, ColorMode, Config, OutputStreamType};
 use crate::highlight::Highlighter;
-use crate::keys::KeyPress;
+use crate::keys::KeyEvent;
 use crate::layout::{Layout, Position};
 use crate::line_buffer::LineBuffer;
-use crate::Result;
+use crate::{Cmd, Result};
 
 /// Terminal state
 pub trait RawMode: Sized {
@@ -18,12 +18,14 @@ pub trait RawMode: Sized {
 /// Translate bytes read from stdin to keys.
 pub trait RawReader {
     /// Blocking read of key pressed.
-    fn next_key(&mut self, single_esc_abort: bool) -> Result<KeyPress>;
+    fn next_key(&mut self, single_esc_abort: bool) -> Result<KeyEvent>;
     /// For CTRL-V support
     #[cfg(unix)]
     fn next_char(&mut self) -> Result<char>;
     /// Bracketed paste
     fn read_pasted_text(&mut self) -> Result<String>;
+    /// Check if `key` is bound to a peculiar command
+    fn find_binding(&self, key: &KeyEvent) -> Option<Cmd>;
 }
 
 /// Display prompt, line and cursor in terminal output
@@ -64,7 +66,7 @@ pub trait Renderer {
             self.calculate_position(&line[pos..], cursor)
         };
         if let Some(info) = info {
-            end = self.calculate_position(&info, end);
+            end = self.calculate_position(info, end);
         }
 
         let new_layout = Layout {
@@ -199,6 +201,7 @@ fn width(s: &str, esc_seq: &mut u8) -> usize {
 
 /// Terminal contract
 pub trait Term {
+    type KeyMap;
     type Reader: RawReader; // rl_instream
     type Writer: Renderer<Reader = Self::Reader>; // rl_outstream
     type Mode: RawMode;
@@ -208,6 +211,7 @@ pub trait Term {
         stream: OutputStreamType,
         tab_stop: usize,
         bell_style: BellStyle,
+        enable_bracketed_paste: bool,
     ) -> Self;
     /// Check if current terminal can provide a rich line-editing user
     /// interface.
@@ -217,9 +221,9 @@ pub trait Term {
     /// check if output stream is connected to a terminal.
     fn is_output_tty(&self) -> bool;
     /// Enable RAW mode for the terminal.
-    fn enable_raw_mode(&mut self) -> Result<Self::Mode>;
+    fn enable_raw_mode(&mut self) -> Result<(Self::Mode, Self::KeyMap)>;
     /// Create a RAW reader
-    fn create_reader(&self, config: &Config) -> Result<Self::Reader>;
+    fn create_reader(&self, config: &Config, key_map: Self::KeyMap) -> Result<Self::Reader>;
     /// Create a writer
     fn create_writer(&self) -> Self::Writer;
 }
