@@ -1,26 +1,27 @@
 //! Windows specific definitions
 #![allow(clippy::try_err)] // suggested fix does not work (cannot infer...)
 
-use std::io::{self, ErrorKind, Write};
+use std::io::{self, Write};
 use std::mem;
-use std::sync::atomic;
+use std::sync::atomic::{AtomicBool, Ordering};
 
 use log::{debug, warn};
 use unicode_segmentation::UnicodeSegmentation;
 use unicode_width::UnicodeWidthStr;
 use winapi::shared::minwindef::{BOOL, DWORD, FALSE, TRUE, WORD};
 use winapi::shared::winerror;
+use winapi::um::handleapi::INVALID_HANDLE_VALUE;
+use winapi::um::wincon::{self, CONSOLE_SCREEN_BUFFER_INFO, COORD};
 use winapi::um::winnt::{CHAR, HANDLE};
-use winapi::um::{consoleapi, handleapi, processenv, winbase, wincon, winuser};
+use winapi::um::{consoleapi, processenv, winbase, winuser};
 
 use super::{width, RawMode, RawReader, Renderer, Term};
 use crate::config::{BellStyle, ColorMode, Config, OutputStreamType};
-use crate::error;
 use crate::highlight::Highlighter;
-use crate::keys::{self, KeyPress};
+use crate::keys::{KeyCode as K, KeyEvent, Modifiers as M};
 use crate::layout::{Layout, Position};
 use crate::line_buffer::LineBuffer;
-use crate::Result;
+use crate::{error, Cmd, Result};
 
 const STDIN_FILENO: DWORD = winbase::STD_INPUT_HANDLE;
 const STDOUT_FILENO: DWORD = winbase::STD_OUTPUT_HANDLE;
@@ -28,7 +29,7 @@ const STDERR_FILENO: DWORD = winbase::STD_ERROR_HANDLE;
 
 fn get_std_handle(fd: DWORD) -> Result<HANDLE> {
     let handle = unsafe { processenv::GetStdHandle(fd) };
-    if handle == handleapi::INVALID_HANDLE_VALUE {
+    if handle == INVALID_HANDLE_VALUE {
         Err(io::Error::last_os_error())?;
     } else if handle.is_null() {
         Err(io::Error::new(
@@ -64,6 +65,11 @@ fn get_console_mode(handle: HANDLE) -> Result<DWORD> {
     Ok(original_mode)
 }
 
+type ConsoleKeyMap = ();
+#[cfg(not(test))]
+pub type KeyMap = ConsoleKeyMap;
+
+#[must_use = "You must restore default mode (disable_raw_mode)"]
 #[cfg(not(test))]
 pub type Mode = ConsoleMode;
 
@@ -101,7 +107,7 @@ impl ConsoleRawReader {
 }
 
 impl RawReader for ConsoleRawReader {
-    fn next_key(&mut self, _: bool) -> Result<KeyPress> {
+    fn next_key(&mut self, _: bool) -> Result<KeyEvent> {
         use std::char::decode_utf16;
         use winapi::um::wincon::{
             LEFT_ALT_PRESSED, LEFT_CTRL_PRESSED, RIGHT_ALT_PRESSED, RIGHT_CTRL_PRESSED,
@@ -113,12 +119,10 @@ impl RawReader for ConsoleRawReader {
         let mut surrogate = 0;
         loop {
             // TODO GetNumberOfConsoleInputEvents
-            check(unsafe {
-                consoleapi::ReadConsoleInputW(self.handle, &mut rec, 1 as DWORD, &mut count)
-            })?;
+            check(unsafe { consoleapi::ReadConsoleInputW(self.handle, &mut rec, 1, &mut count) })?;
 
             if rec.EventType == wincon::WINDOW_BUFFER_SIZE_EVENT {
-                SIGWINCH.store(true, atomic::Ordering::SeqCst);
+                SIGWINCH.store(true, Ordering::SeqCst);
                 debug!(target: "rustyline", "SIGWINCH");
                 return Err(error::ReadlineError::WindowResize); // sigwinch +
                                                                 // err => err
@@ -136,76 +140,69 @@ impl RawReader for ConsoleRawReader {
 
             let alt_gr = key_event.dwControlKeyState & (LEFT_CTRL_PRESSED | RIGHT_ALT_PRESSED)
                 == (LEFT_CTRL_PRESSED | RIGHT_ALT_PRESSED);
-            let alt = key_event.dwControlKeyState & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED) != 0;
-            let ctrl = key_event.dwControlKeyState & (LEFT_CTRL_PRESSED | RIGHT_CTRL_PRESSED) != 0;
-            let meta = alt && !alt_gr;
-            let shift = key_event.dwControlKeyState & SHIFT_PRESSED != 0;
+            let mut mods = M::NONE;
+            if !alt_gr
+                && key_event.dwControlKeyState & (LEFT_CTRL_PRESSED | RIGHT_CTRL_PRESSED) != 0
+            {
+                mods |= M::CTRL;
+            }
+            if !alt_gr && key_event.dwControlKeyState & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED) != 0
+            {
+                mods |= M::ALT;
+            }
+            if key_event.dwControlKeyState & SHIFT_PRESSED != 0 {
+                mods |= M::SHIFT;
+            }
 
             let utf16 = unsafe { *key_event.uChar.UnicodeChar() };
-            if utf16 == 0 {
-                match i32::from(key_event.wVirtualKeyCode) {
-                    winuser::VK_LEFT => {
-                        return Ok(if ctrl {
-                            KeyPress::ControlLeft
-                        } else if shift {
-                            KeyPress::ShiftLeft
-                        } else {
-                            KeyPress::Left
-                        });
+            let key_code = match i32::from(key_event.wVirtualKeyCode) {
+                winuser::VK_LEFT => K::Left,
+                winuser::VK_RIGHT => K::Right,
+                winuser::VK_UP => K::Up,
+                winuser::VK_DOWN => K::Down,
+                winuser::VK_DELETE => K::Delete,
+                winuser::VK_HOME => K::Home,
+                winuser::VK_END => K::End,
+                winuser::VK_PRIOR => K::PageUp,
+                winuser::VK_NEXT => K::PageDown,
+                winuser::VK_INSERT => K::Insert,
+                winuser::VK_F1 => K::F(1),
+                winuser::VK_F2 => K::F(2),
+                winuser::VK_F3 => K::F(3),
+                winuser::VK_F4 => K::F(4),
+                winuser::VK_F5 => K::F(5),
+                winuser::VK_F6 => K::F(6),
+                winuser::VK_F7 => K::F(7),
+                winuser::VK_F8 => K::F(8),
+                winuser::VK_F9 => K::F(9),
+                winuser::VK_F10 => K::F(10),
+                winuser::VK_F11 => K::F(11),
+                winuser::VK_F12 => K::F(12),
+                winuser::VK_BACK => K::Backspace, // vs Ctrl-h
+                winuser::VK_RETURN => K::Enter,   // vs Ctrl-m
+                winuser::VK_ESCAPE => K::Esc,
+                winuser::VK_TAB => {
+                    if mods.contains(M::SHIFT) {
+                        mods.remove(M::SHIFT);
+                        K::BackTab
+                    } else {
+                        K::Tab // vs Ctrl-i
                     }
-                    winuser::VK_RIGHT => {
-                        return Ok(if ctrl {
-                            KeyPress::ControlRight
-                        } else if shift {
-                            KeyPress::ShiftRight
-                        } else {
-                            KeyPress::Right
-                        });
+                }
+                _ => {
+                    if utf16 == 0 {
+                        continue;
+                    } else {
+                        K::UnknownEscSeq
                     }
-                    winuser::VK_UP => {
-                        return Ok(if ctrl {
-                            KeyPress::ControlUp
-                        } else if shift {
-                            KeyPress::ShiftUp
-                        } else {
-                            KeyPress::Up
-                        });
-                    }
-                    winuser::VK_DOWN => {
-                        return Ok(if ctrl {
-                            KeyPress::ControlDown
-                        } else if shift {
-                            KeyPress::ShiftDown
-                        } else {
-                            KeyPress::Down
-                        });
-                    }
-                    winuser::VK_DELETE => return Ok(KeyPress::Delete),
-                    winuser::VK_HOME => return Ok(KeyPress::Home),
-                    winuser::VK_END => return Ok(KeyPress::End),
-                    winuser::VK_PRIOR => return Ok(KeyPress::PageUp),
-                    winuser::VK_NEXT => return Ok(KeyPress::PageDown),
-                    winuser::VK_INSERT => return Ok(KeyPress::Insert),
-                    winuser::VK_F1 => return Ok(KeyPress::F(1)),
-                    winuser::VK_F2 => return Ok(KeyPress::F(2)),
-                    winuser::VK_F3 => return Ok(KeyPress::F(3)),
-                    winuser::VK_F4 => return Ok(KeyPress::F(4)),
-                    winuser::VK_F5 => return Ok(KeyPress::F(5)),
-                    winuser::VK_F6 => return Ok(KeyPress::F(6)),
-                    winuser::VK_F7 => return Ok(KeyPress::F(7)),
-                    winuser::VK_F8 => return Ok(KeyPress::F(8)),
-                    winuser::VK_F9 => return Ok(KeyPress::F(9)),
-                    winuser::VK_F10 => return Ok(KeyPress::F(10)),
-                    winuser::VK_F11 => return Ok(KeyPress::F(11)),
-                    winuser::VK_F12 => return Ok(KeyPress::F(12)),
-                    // winuser::VK_BACK is correctly handled because the key_event.UnicodeChar is
-                    // also set.
-                    _ => continue,
-                };
+                }
+            };
+            let key = if key_code != K::UnknownEscSeq {
+                KeyEvent(key_code, mods)
             } else if utf16 == 27 {
-                return Ok(KeyPress::Esc);
+                KeyEvent(K::Esc, mods) // FIXME dead code ?
             } else {
-                if utf16 >= 0xD800 && utf16 < 0xDC00 {
+                if (0xD800..0xDC00).contains(&utf16) {
                     surrogate = utf16;
                     continue;
                 }
@@ -220,23 +217,19 @@ impl RawReader for ConsoleRawReader {
                     return Err(error::ReadlineError::Eof);
                 };
                 let c = rc?;
-                if meta {
-                    return Ok(KeyPress::Meta(c));
-                } else {
-                    let mut key = keys::char_to_key_press(c);
-                    if key == KeyPress::Tab && shift {
-                        key = KeyPress::BackTab;
-                    } else if key == KeyPress::Char(' ') && ctrl {
-                        key = KeyPress::Ctrl(' ');
-                    }
-                    return Ok(key);
-                }
-            }
+                KeyEvent::new(c, mods)
+            };
+            debug!(target: "rustyline", "wVirtualKeyCode: {:#x}, utf16: {:#x}, dwControlKeyState: {:#x} => key: {:?}", key_event.wVirtualKeyCode, utf16, key_event.dwControlKeyState, key);
+            return Ok(key);
         }
     }
 
     fn read_pasted_text(&mut self) -> Result<String> {
-        unimplemented!()
+        Ok(clipboard_win::get_clipboard_string()?)
+    }
+
+    fn find_binding(&self, _: &KeyEvent) -> Option<Cmd> {
+        None
     }
 }
 
@@ -268,17 +261,17 @@ impl ConsoleRenderer {
         }
     }
 
-    fn get_console_screen_buffer_info(&self) -> Result<wincon::CONSOLE_SCREEN_BUFFER_INFO> {
+    fn get_console_screen_buffer_info(&self) -> Result<CONSOLE_SCREEN_BUFFER_INFO> {
         let mut info = unsafe { mem::zeroed() };
         check(unsafe { wincon::GetConsoleScreenBufferInfo(self.handle, &mut info) })?;
         Ok(info)
     }
 
-    fn set_console_cursor_position(&mut self, pos: wincon::COORD) -> Result<()> {
+    fn set_console_cursor_position(&mut self, pos: COORD) -> Result<()> {
         check(unsafe { wincon::SetConsoleCursorPosition(self.handle, pos) })
     }
 
-    fn clear(&mut self, length: DWORD, pos: wincon::COORD, attr: WORD) -> Result<()> {
+    fn clear(&mut self, length: DWORD, pos: COORD, attr: WORD) -> Result<()> {
         let mut _count = 0;
         check(unsafe {
             wincon::FillConsoleOutputCharacterA(self.handle, ' ' as CHAR, length, pos, &mut _count)
@@ -299,7 +292,6 @@ impl ConsoleRenderer {
         for c in s.graphemes(true) {
             if c == "\n" {
                 col = 0;
-                self.buffer.push_str(c);
             } else {
                 let cw = width(c, &mut esc_seq);
                 col += cw;
@@ -307,8 +299,8 @@ impl ConsoleRenderer {
                     self.buffer.push('\n');
                     col = cw;
                 }
-                self.buffer.push_str(c);
             }
+            self.buffer.push_str(c);
         }
         if col == self.cols {
             self.buffer.push('\n');
@@ -318,11 +310,7 @@ impl ConsoleRenderer {
     }
 
     // position at the start of the prompt, clear to end of previous input
-    fn clear_old_rows(
-        &mut self,
-        info: &wincon::CONSOLE_SCREEN_BUFFER_INFO,
-        layout: &Layout,
-    ) -> Result<()> {
+    fn clear_old_rows(&mut self, info: &CONSOLE_SCREEN_BUFFER_INFO, layout: &Layout) -> Result<()> {
         let current_row = layout.cursor.row;
         let old_rows = layout.end.row;
         let mut coord = info.dwCursorPosition;
@@ -471,14 +459,16 @@ impl Renderer for ConsoleRenderer {
     /// Clear the screen. Used to handle ctrl+l
     fn clear_screen(&mut self) -> Result<()> {
         let info = self.get_console_screen_buffer_info()?;
-        let coord = wincon::COORD { X: 0, Y: 0 };
+        let coord = COORD { X: 0, Y: 0 };
         check(unsafe { wincon::SetConsoleCursorPosition(self.handle, coord) })?;
         let n = info.dwSize.X as DWORD * info.dwSize.Y as DWORD;
         self.clear(n, coord, info.wAttributes)
     }
 
     fn sigwinch(&self) -> bool {
-        SIGWINCH.compare_and_swap(true, false, atomic::Ordering::SeqCst)
+        SIGWINCH
+            .compare_exchange(true, false, Ordering::SeqCst, Ordering::SeqCst)
+            .unwrap_or(false)
     }
 
     /// Try to get the number of columns in the current terminal,
@@ -524,7 +514,7 @@ impl Renderer for ConsoleRenderer {
     }
 }
 
-static SIGWINCH: atomic::AtomicBool = atomic::AtomicBool::new(false);
+static SIGWINCH: AtomicBool = AtomicBool::new(false);
 
 #[cfg(not(test))]
 pub type Terminal = Console;
@@ -553,6 +543,7 @@ impl Console {
 }
 
 impl Term for Console {
+    type KeyMap = ConsoleKeyMap;
     type Mode = ConsoleMode;
     type Reader = ConsoleRawReader;
     type Writer = ConsoleRenderer;
@@ -562,6 +553,7 @@ impl Term for Console {
         stream_type: OutputStreamType,
         _tab_stop: usize,
         bell_style: BellStyle,
+        _enable_bracketed_paste: bool,
     ) -> Console {
         use std::ptr;
         let stdin_handle = get_std_handle(STDIN_FILENO);
@@ -616,7 +608,7 @@ impl Term for Console {
     // }
 
     /// Enable RAW mode for the terminal.
-    fn enable_raw_mode(&mut self) -> Result<Self::Mode> {
+    fn enable_raw_mode(&mut self) -> Result<(ConsoleMode, ConsoleKeyMap)> {
         if !self.stdin_isatty {
             Err(io::Error::new(
                 io::ErrorKind::Other,
@@ -644,7 +636,7 @@ impl Term for Console {
                 mode |= wincon::ENABLE_WRAP_AT_EOL_OUTPUT;
                 debug!(target: "rustyline", "activate ENABLE_WRAP_AT_EOL_OUTPUT");
                 unsafe {
-                    assert!(consoleapi::SetConsoleMode(self.stdstream_handle, mode) != 0);
+                    assert_ne!(consoleapi::SetConsoleMode(self.stdstream_handle, mode), 0);
                 }
             }
             // To enable ANSI colors (Windows 10 only):
@@ -655,7 +647,7 @@ impl Term for Console {
                     mode &= !wincon::ENABLE_VIRTUAL_TERMINAL_PROCESSING;
                     debug!(target: "rustyline", "deactivate ENABLE_VIRTUAL_TERMINAL_PROCESSING");
                     unsafe {
-                        assert!(consoleapi::SetConsoleMode(self.stdstream_handle, mode) != 0);
+                        assert_ne!(consoleapi::SetConsoleMode(self.stdstream_handle, mode), 0);
                     }
                 } else {
                     debug!(target: "rustyline", "ANSI colors already enabled");
@@ -671,15 +663,18 @@ impl Term for Console {
             None
         };
 
-        Ok(ConsoleMode {
-            original_stdin_mode,
-            stdin_handle: self.stdin_handle,
-            original_stdstream_mode,
-            stdstream_handle: self.stdstream_handle,
-        })
+        Ok((
+            ConsoleMode {
+                original_stdin_mode,
+                stdin_handle: self.stdin_handle,
+                original_stdstream_mode,
+                stdstream_handle: self.stdstream_handle,
+            },
+            (),
+        ))
     }
 
-    fn create_reader(&self, _: &Config) -> Result<ConsoleRawReader> {
+    fn create_reader(&self, _: &Config, _: ConsoleKeyMap) -> Result<ConsoleRawReader> {
         ConsoleRawReader::create()
     }
 
